#!/usr/local/bin/perl
#
# 
# Mu-Cheng Hsieh <mucheng@cc.nthu.edu.tw>
# 2003/10/10
#

($router_ip, $router_type, $router_login, $router_password, $router_comment);
$sock;	# current socket

$SIG{INT} = \&my_exit;
$SIG{KILL} = \&my_exit;
$SIG{TERM} = \&my_exit;

#
##############################################################################
#
# Subroutines
#
##############################################################################
sub check_ip_format{
  my ($ip) = @_;
  #print STDOUT "check_ip_format: $ip\r\n";
  my $i;
  if ($ip =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
    foreach $i ($1, $2, $3, $4) {
      if ($i < 0 || $i > 255 ) {
        return -1;
      }
    }
  } else {
    return -1;
  }
  return 0;
}

sub my_exit{
  &logout_router() if ($sock);
  exit(); 
}

sub telnet_err{
  my ($message) = @_;
  print "\ntelnet_err: $message\n";
  unless ($message =~ /handle is closed/) {
    &logout_router();
  }
  return -1;
}

sub login_router{

  ($router_type, $router_login, $router_password, $router_comment)
  = split /$delimiter/, $ROUTERS{$router_ip};

  $sock = Net::Telnet->new(Host => "$router_ip",
                                  Timeout => 10, 
                                  #Dump_Log => "dumplog", 
                                  #Input_log => "inputlog", 
                                  #Output_log => "outputlog", 
                                  Errmode => \&telnet_err);

  my $len=1024*1024*64;
  $sock->max_buffer_length($len);

  if ( $router_type eq 'extreme' ) {
    $sock->login($router_login, $router_password);
    $sock->prompt('/ # $/');
    $sock->cmd('disable clipaging');
  } elsif ( $router_type eq 'cisco' ) {
    $sock->waitfor('/Password: $/'); 	# User Access Verification
    $sock->print($router_login);
    $sock->waitfor('/>$/'); 
    $sock->print('enable');
    $sock->waitfor('/Password: $/'); 
    $sock->print($router_password);
    $sock->waitfor('/#$/'); 
    $sock->prompt('/#$/');
    $sock->cmd('terminal length 0');
  }
  return 0;
}

sub sock_cmd{
  my ($command) = @_;
  my @lines = ();
  @lines = $sock->cmd("$command"); 
  my $line = join "", @lines;
  return $line;
}

sub logout_router{
  if ( $sock ) {
    if ( $router_type eq 'extreme' ) {
      $sock->cmd('enable clipaging');
      $sock->print('exit');
      $sock->print('n');
    } elsif ( $router_type eq 'cisco' ) {
      $sock->cmd('terminal length 22');
      $sock->print('exit');
    } 
    $sock->close;
  }
}

##############################################################################
#
# search the nearest router among the controllable routers by the target ip
#
##############################################################################
sub search_router{
  my ($target) = @_;
  my ($status, $next_hop, $vlan);
  my $previous_hop = "null"; 

  # the initial router
  $router_ip = (keys %ROUTERS) [0];

 ($router_type, $router_login, $router_password, $router_comment) 
  = split /$delimiter/, $ROUTERS{$router_ip};

  return -1 if ( &login_router ); # login router failure

NEXT_HOP:
  ($status, $next_hop, $vlan) =  check_router($target);
  #print "($status, $next_hop, $vlan)\n";

  if ( $status eq "NEXT" ) {
    if ( defined $ROUTERS{$next_hop} ){

      if ( $next_hop eq $previous_hop ) {	# check loop routing
        print "Loop routing: $previous_hop -> $router_ip -> $next_hop\n";
        return -2;
      }

      # ready to login next hop
      &logout_router;
      $previous_hop = $router_ip;	# remeber the previous hop
      $router_ip = $next_hop;		# change to next hop
      ($router_type, $router_login, $router_password, $router_comment) 
      = split /$delimiter/, $ROUTERS{$router_ip};
      return -1 if (&login_router);
      goto NEXT_HOP;
    }
    print "The most nearest router under control\n";
    return $status, $next_hop, $vlan;
  } elsif ( $status eq "DIRECT" ) {
    print "$status, $next_hop, $vlan\n";
    return $status, $next_hop, $vlan;
  } else {
    print "($status, $next_hop, $vlan)\n";
    return -2;
  }
}

##############################################################################
#
# check the routing status of the target ip in the current router 
#
##############################################################################
sub check_router{
  my ($target) = @_;

  if ( $router_type eq 'extreme' ) {
    return &check_router_extreme($target);
  } elsif ( $router_type eq 'cisco' ) {
    return &check_router_cisco($target);
  }
}

sub check_router_extreme{
  my ($target) = @_;
  my $line;
  my $next_hop;	# next hop router ip
  my $vlan;	# interface or vlan

  $line = sock_cmd("rtlookup $target"); 

#* CoreSwitch-1:5 # rtlookup 163.28.25.1
# *oa 140.114.63.0/24    140.114.1.2     2   UG-----um To-Core2 
# *s  140.113.0.0/16     140.113.251.113 1   UG---S-um nctu   
# *d  140.114.1.4/30     140.114.1.5     1   U------um To-Edge1  
# *r  Default Route      140.114.1.129   2   UG-----um To-M160   
# b - Black hole, be - EBGP, bi - IBGP, bo - BOOTP, ct - CBT, d - Direct
# df - DownIF, dv - DVMRP, h - Hardcoded, i - ICMP, mo - MOSPF, o - OSPF
# oa - OSPFIntra, or - OSPFInter, oe - OSPFAsExt, o1 - OSPFExt1, o2 - OSPFExt2
# pd - PIM-DM, ps - PIM-SM, r - RIP, ra - RtAdvrt, s - Static,
# sv - SLB_VIP, un - UnKnown.

  if ( $line =~ /\s+(\d+\.\d+\.\d+\.\d+)\s+\d+\s+\S+\s+(\S+)\s/ ) {
    $next_hop = $1;  $vlan = $2;
    if ($line =~ /\*d/) {
      return "DIRECT", $router_ip, $vlan;
    } else {
      return "NEXT", $next_hop, $vlan;
    } 
  }
  return "UNKNOWN",$router_ip,$line;
}

sub check_router_cisco{
  my ($target) = @_;
  my $line;
  my $next_hop;	# next hop router ip
  my $vlan;	# interface or vlan

#% Subnet not in table
#% Network not in table
  $line = sock_cmd("show ip route $target");
  if ( $line =~ / not in table/) {

    $line = sock_cmd("show ip route 0.0.0.0");
    if ( $line =~ /\*\s+(\d+\.\d+\.\d+\.\d+)/ ) {
#  Routing Descriptor Blocks:
#  * 140.114.240.254
      $next_hop = $1;

      $line = sock_cmd("show ip route $next_hop");
      if ( $line =~ /\* directly connected, via (\S+)/ ) {
#  * directly connected, via FastEthernet3/1
        return "NEXT",$next_hop, $1;
      } 
    }

  } elsif ( $line =~ /\*\s+(\d+\.\d+\.\d+\.\d+)\,.*via (\S+)/ ) {
#  Routing Descriptor Blocks:
#  * 140.114.241.2, from 140.114.241.2, 00:00:00 ago, via FastEthernet3/2
    return "NEXT",$1,$2;

  } elsif ( $line =~ /\* directly connected, via (\S+)/ ) {
#   Routing Descriptor Blocks:
#   * directly connected, via Vlan243
#   * directly connected, via FastEthernet6/3
    return "DIRECT",$router_ip, $1;
  } else {
    print "Unknown status: [$line]\n";
    return "UNKNOWN",$router_ip,$line;
  }
  return "UNKNOWN",$router_ip,$line;
}

1;
