#!/usr/bin/perl
#
# 
# Mu-Cheng Hsieh <mucheng@cc.nthu.edu.tw>
# 2003/10/13

require "/usr/local/netguard/etc/netguard.conf";

#use constant DEBUG => 1;
use constant DEBUG => 0;

my ($my_timestamp, $my_date, $my_ymd, $my_hms, $my_time);
my ($router_ip, $router_type, $router_login, $router_password, $router_comment);
my $sock;	# current socket
my %REQUEST = ();
my $request_file;
my %BLOCK = ();	# block information
my $block_file;

$SIG{INT} = \&my_exit;
$SIG{KILL} = \&my_exit;
$SIG{TERM} = \&my_exit;

# main
if ( -f "$log_dir/netguard.pid" ) {
  print "Error: another $0 is running!\n";
  exit;
}

&check_dirs;

&write2file("$log_dir/netguard.pid", "$$\n");

while(1){
  open RQ_FH, "/bin/ls $request_dir|";
  while(<RQ_FH>){
    chop;
    reply_request("$request_dir/$_");
  }
  close RQ_FH;
  sleep 30;
}

my_exit;

#
##############################################################################
#
# Subroutines
#
##############################################################################
sub check_ip_format{
  my ($ip) = @_;
  #print STDOUT "check_ip_format: $ip\r\n";
  my $i;
  if ($ip =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
    foreach $i ($1, $2, $3, $4) {
      if ($i < 0 || $i > 255 ) {
        return -1;
      }
    }
  } else {
    return -1;
  }
  return 0;
}

sub reply_request{
  %REQUEST = (); 
  ($request_file) = @_;

  # request parser
  open FH, $request_file;
  while(<FH>){
    chop;
    my ($key, $value) = split /: /;
    $REQUEST{$key} = $value;
    print "$key -> $value\n";
  }
  close FH;

  # run request
  &get_time;
  &write2file("$request_file", "ACTION_TIME: $my_time\n");
  &write2file("$request_file", "ACTION_TIMESTAMP: $my_timestamp\n");

  # record the action times
  my $action_times;
  if ( defined $REQUEST{'ACTION_TIMES'} ) {
    $action_times =  $REQUEST{'ACTION_TIMES'} + 1;
  } else {
    $action_times = 1;
  }

  &write2file("$request_file", "ACTION_TIMES: $action_times\n");

  # check the action times
  if ( $action_times > 3 ) {
    &write2file("$request_file", "ACTION_RESULT: over 3 times\n");
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
    return -1;
  }

  # block ip
  if ($REQUEST{'REQUEST_ACTION'} eq "BLOCK_IP") {
    print "$REQUEST{'REQUEST_ACTION'} $REQUEST{'REQUEST_TARGET'}\n"; 
    # check ip format
    if ( check_ip_format($REQUEST{'REQUEST_TARGET'}) ){
      &write2file("$request_file", "ACTION_RESULT: invalid target ip\n");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
      return -2;
    }

    # duplicate block ip request
    if ( -f "$block_dir/$REQUEST{'REQUEST_TARGET'}" ) {
      &write2file("$request_file", "ACTION_RESULT: already be blocked\n");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},D";
      return -1;
    }

    return &block_ip($REQUEST{'REQUEST_TARGET'});

  # unblock ip
  } elsif ($REQUEST{'REQUEST_ACTION'} eq "UNBLOCK_IP") {
    print "$REQUEST{'REQUEST_ACTION'} $REQUEST{'REQUEST_TARGET'}\n"; 
    # check ip format
    if ( check_ip_format($REQUEST{'REQUEST_TARGET'}) ){
      &write2file("$request_file", "ACTION_RESULT: invalid target ip\n");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
      return -2;
    }

    # invalid unblock ip request
    if ( ! -f "$block_dir/$REQUEST{'REQUEST_TARGET'}" ) {
      &write2file("$request_file", "ACTION_RESULT: $REQUEST{'REQUEST_TARGET'} not be blocked\n");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},D";
      return -1;
    }

    return &unblock_ip($REQUEST{'REQUEST_TARGET'});

  } else {
    &write2file("$request_file", "ACTION_RESULT: unknown request action\n");
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
    return -2;
  }
}

sub check_dirs{
  my $dir;
  foreach $dir ($spool_dir, $request_dir, $block_dir, $unblock_dir, $archive_dir, $info_dir, $log_dir) { 
    if ( -d $dir ) {
      print "Check directory $dir: OK\n";
    } else {
      if ( mkdir $dir ) {
        print "Make directory $dir: OK\n";
      } else {
        print "Make directory $dir: Fail\n";
      }
    }
  } 
}

sub write2file{
  my ($file, $data) = @_; 
  open FH, ">> $file";
  #&get_time;
  #print FH "$my_date $my_hms\n$data";
  print FH "$data";
  close FH;
}

sub debug{
  if (DEBUG) {
    &get_time;
    open DF, ">> $debug_file";
    print DF "$my_date $my_hms @_";
    close DF;
  }
}

sub get_time{
  use Time::HiRes qw(gettimeofday);
  my ($seconds, $microseconds) = split /\./, gettimeofday;
  $my_timestamp = $seconds;
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst)
     = localtime ($my_timestamp);
  $year+=1900;
  $mon++;
  $my_date = sprintf("%04d/%02d/%02d", $year, $mon, $mday);
  $my_ymd = sprintf("%04d%02d%02d", $year, $mon, $mday);
  $my_hms = sprintf("%02d:%02d:%02d", $hour, $min, $sec);
  $my_time = sprintf("%04d%02d%02d.%8s.%s", $year, $mon, $mday, $my_hms, $microseconds);
}

sub my_return{
  &logout_router() if ($sock);
}

sub my_exit{
  &logout_router() if ($sock);
  exit(); 
}

sub telnet_err{
  my ($message) = @_;
  print "\ntelnet_err: $message\n";
  write2file("$request_file", "ERROR: login_router $message\n");
  unless ($message =~ /handle is closed/) {
    &logout_router();
  }
  return -1;
}

sub login_router{
  debug("login_router: $router_ip, $router_comment\n");

  $sock = Net::Telnet->new(Host => "$router_ip",
                                  Timeout => 10, 
                                  #Dump_Log => "dumplog", 
                                  #Input_log => "inputlog", 
                                  #Output_log => "outputlog", 
                                  Errmode => \&telnet_err);

  if ( $router_type eq 'extreme' ) {
    $sock->login($router_login, $router_password);
    $sock->prompt('/ # $/');
    $sock->cmd('disable clipaging');
  } elsif ( $router_type eq 'cisco' ) {
    $sock->waitfor('/Password: $/'); 	# User Access Verification
    $sock->print($router_login);
    $sock->waitfor('/>$/'); 
    $sock->print('enable');
    $sock->waitfor('/Password: $/'); 
    $sock->print($router_password);
    $sock->waitfor('/#$/'); 
    $sock->prompt('/#$/');
    $sock->cmd('terminal length 0');
  }
  return 0;
}

sub sock_cmd{
  my ($command) = @_;
  my @lines = ();
  @lines = $sock->cmd("$command"); 
  my $line = join "", @lines;
  return $line;
}

sub logout_router{
  if ( $sock ) {
    debug("logout_router: $router_ip, $router_comment\n");
    if ( $router_type eq 'extreme' ) {
      $sock->cmd('enable clipaging');
      $sock->print('exit');
      $sock->print('n');
    } elsif ( $router_type eq 'cisco' ) {
      $sock->cmd('terminal length 22');
      $sock->print('exit');
    } 
    $sock->close;
  }
}

##############################################################################
#
# search the nearest router among the controllable routers by the target ip
#
##############################################################################
sub search_router{
  my ($target) = @_;
  my ($status, $next_hop, $vlan);
  my $previous_hop = "null"; 

  # the initial router
  $router_ip = (keys %ROUTERS) [0];

 ($router_type, $router_login, $router_password, $router_comment) 
  = split /$delimiter/, $ROUTERS{$router_ip};

  return -1 if ( &login_router ); # login router failure

NEXT_HOP:
  ($status, $next_hop, $vlan) =  check_router($target);
  #print "($status, $next_hop, $vlan)\n";

  if ( $status eq "NEXT" ) {
    if ( defined $ROUTERS{$next_hop} ){

      if ( $next_hop eq $previous_hop ) {	# check loop routing
        print "Loop routing: $previous_hop -> $router_ip -> $next_hop\n";
        write2file("$request_file", "ERROR: infinite routing loop $previous_hop -> $router_ip -> $next_hop\n");
        return -2;
      }

      # ready to login next hop
      &logout_router;
      $previous_hop = $router_ip;	# remeber the previous hop
      $router_ip = $next_hop;		# change to next hop
      ($router_type, $router_login, $router_password, $router_comment) 
      = split /$delimiter/, $ROUTERS{$router_ip};
      return -1 if (&login_router);
      goto NEXT_HOP;
    }
    print "The most nearest router under control\n";
    return $status, $next_hop, $vlan;
  } elsif ( $status eq "DIRECT" ) {
    print "$status, $next_hop, $vlan\n";
    return $status, $next_hop, $vlan;
  } else {
    print "($status, $next_hop, $vlan)\n";
    return -2;
  }
}

##############################################################################
#
# check the routing status of the target ip in the current router 
#
##############################################################################
sub check_router{
  my ($target) = @_;

  if ( $router_type eq 'extreme' ) {
    return &check_router_extreme($target);
  } elsif ( $router_type eq 'cisco' ) {
    return &check_router_cisco($target);
  }
}

sub check_router_extreme{
  my ($target) = @_;
  my $line;
  my $next_hop;	# next hop router ip
  my $vlan;	# interface or vlan

  $line = sock_cmd("rtlookup $target"); 

#* CoreSwitch-1:5 # rtlookup 163.28.25.1
# *oa 140.114.63.0/24    140.114.1.2     2   UG-----um To-Core2 
# *s  140.113.0.0/16     140.113.251.113 1   UG---S-um nctu   
# *d  140.114.1.4/30     140.114.1.5     1   U------um To-Edge1  
# *r  Default Route      140.114.1.129   2   UG-----um To-M160   
# b - Black hole, be - EBGP, bi - IBGP, bo - BOOTP, ct - CBT, d - Direct
# df - DownIF, dv - DVMRP, h - Hardcoded, i - ICMP, mo - MOSPF, o - OSPF
# oa - OSPFIntra, or - OSPFInter, oe - OSPFAsExt, o1 - OSPFExt1, o2 - OSPFExt2
# pd - PIM-DM, ps - PIM-SM, r - RIP, ra - RtAdvrt, s - Static,
# sv - SLB_VIP, un - UnKnown.

  if ( $line =~ /\s+(\d+\.\d+\.\d+\.\d+)\s+\d+\s+\S+\s+(\S+)\s/ ) {
    $next_hop = $1;  $vlan = $2;
    if ($line =~ /\*d/) {
      return "DIRECT", $router_ip, $vlan;
    } else {
      return "NEXT", $next_hop, $vlan;
    } 
  }
  return "UNKNOWN",$router_ip,$line;
}

sub check_router_cisco{
  my ($target) = @_;
  my $line;
  my $next_hop;	# next hop router ip
  my $vlan;	# interface or vlan

#% Subnet not in table
#% Network not in table
  $line = sock_cmd("show ip route $target");
  if ( $line =~ / not in table/) {

    $line = sock_cmd("show ip route 0.0.0.0");
    if ( $line =~ /\*\s+(\d+\.\d+\.\d+\.\d+)/ ) {
#  Routing Descriptor Blocks:
#  * 140.114.240.254
      $next_hop = $1;

      $line = sock_cmd("show ip route $next_hop");
      if ( $line =~ /\* directly connected, via (\S+)/ ) {
#  * directly connected, via FastEthernet3/1
        return "NEXT",$next_hop, $1;
      } 
    }

  } elsif ( $line =~ /\*\s+(\d+\.\d+\.\d+\.\d+)\,.*via (\S+)/ ) {
#  Routing Descriptor Blocks:
#  * 140.114.241.2, from 140.114.241.2, 00:00:00 ago, via FastEthernet3/2
    return "NEXT",$1,$2;

  } elsif ( $line =~ /\* directly connected, via (\S+)/ ) {
#   Routing Descriptor Blocks:
#   * directly connected, via Vlan243
#   * directly connected, via FastEthernet6/3
    return "DIRECT",$router_ip, $1;
  } else {
    print "Unknown status: [$line]\n";
    return "UNKNOWN",$router_ip,$line;
  }
  return "UNKNOWN",$router_ip,$line;
}

##############################################################################
#
# block/unblock ip/network
#
##############################################################################
sub block_ip{
  my ($ip) = @_;

  debug("block_ip: $ip\n");

  my ($status, $next_hop, $vlan) = &search_router($ip);
  if ($status == -1) {
    return -1;
  } elsif ($status == -2 ) {
    return -2
  } 

  &write2file("$request_file", "ROUTER_IP: $router_ip\n");
  &write2file("$request_file", "ROUTER_TYPE: $router_type\n");

  # for extreme
  if ($router_type eq 'extreme') {
    my $ret1 = &block_mac_extreme($ip, $vlan);
    &write2file("$request_file", "BLOCK_MAC_RESULT: $ret1\n");

    my $ret2 = &block_ip_extreme($ip);
    &write2file("$request_file", "BLOCK_IP_RESULT: $ret2\n");

    # Just one condition is enough
    if ( $ret1 == 1 || $ret2 == 1 ) {
      &get_time;
      &write2file("$request_file", "BLOCK_IP_TIME: $my_time\n");
      &write2file("$request_file", "BLOCK_IP_TIMESTAMP: $my_timestamp\n");
      system("/bin/cp $request_file $block_dir/$ip");
      system("/bin/cp $request_file $info_dir/$ip");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},S";
    } 

  } elsif ( $router_type eq 'cisco' ) {
    &block_mac_cisco($ip);
    &block_ip_cisco($ip, $vlan);
  }
}

sub unblock_ip{
  my ($ip) = @_;

  %BLOCK = (); # reset
  # check block
  $block_file = "$block_dir/$ip";
  if ( -f $block_file ){
    open FH, $block_file;
    while(<FH>){
      chop;
      my ($key, $value) = split /: /;
      $BLOCK{$key} = $value;
      print "$key -> $value\n";
    }
    close FH; 
  } else {
    &write2file("$request_file", "UNBLOCK_IP_STATUS: $block_file not found.\n");
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
    return -1;
  }

  $router_ip = $BLOCK{'ROUTER_IP'};

  ($router_type, $router_login, $router_password, $router_comment) 
  = split /$delimiter/, $ROUTERS{$router_ip};

  return -1 if ( &login_router ); # login router failure

  if ($router_type eq 'extreme') {
    my $ret1 = &unblock_mac_extreme;
    my $ret2 = &unblock_ip_extreme;

    # Two conditions need be at the same time
    if ($ret1 == 1 && $ret2 == 1 ) {
      &get_time;
      &write2file("$block_file", "UNBLOCK_IP_TIME: $my_time\n");
      &write2file("$block_file", "UNBLOCK_IP_TIMESTAMP: $my_timestamp\n");
      system("/bin/mv $block_file $unblock_dir/$BLOCK{'REQUEST_TARGET'},$my_time");
      &write2file("$request_file", "UNBLOCK_IP_TIME: $my_time\n");
      &write2file("$request_file", "UNBLOCK_IP_TIMESTAMP: $my_timestamp\n");
      rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},S";
    }
  } elsif ( $router_type eq 'cisco' ) {
    &unblock_mac_cisco;
    &unblock_ip_cisco;
  }
}

sub block_mac_cisco{
  my ($ip) = @_;
  my $line;
  my $mac;
  my $vlan_id;

#sw2>show ip arp 140.114.244.100
#Protocol  Address          Age (min)  Hardware Addr   Type   Interface
#Internet  140.114.244.100        33   0002.b302.f715  ARPA   FastEthernet6/3
  $line = sock_cmd("show ip arp $ip"); 
  if ( $line =~ /\s+(\S+)\s+ARPA\s+/ ) {
    $mac = $1;
    &write2file("$request_file", "TARGET_MAC: $mac\n");
  } else {
    &write2file("$request_file", "BLOCK_MAC_STATUS: MAC not found\n");
    return -1;
  }

#sw2#show mac-address-table address 0002.b302.f715
#  vlan   mac address     type    learn qos            ports
#------+----------------+--------+-----+---+--------------------------
#   243  0002.b302.f715   dynamic  No    --  Fa6/4
  $line = sock_cmd("show mac-address-table address $mac");
  if ( $line =~ /\s+(\d+)\s+$mac\s+/ ) {
    $vlan_id = $1;
    &write2file("$request_file", "TARGET_VLAN_ID: $vlan_id\n");

    $sock->cmd("config terminal");
    my $command = "mac-address-table static $mac vlan $vlan_id drop";
    &write2file("$request_file", "BLOCK_MAC_COMMAND: $command\n");
    $line = sock_cmd("$command");
    &write2file("$request_file", "BLOCK_MAC_STATUS: $line\n");
    $sock->cmd("exit");
  } else {
    &write2file("$request_file", "BLOCK_MAC_STATUS: vlan-id not found:$line\n");
    return -1;
  }
}

sub unblock_mac_cisco{
  unless ( defined $BLOCK{'BLOCK_MAC_COMMAND'} ) {
    &write2file("$request_file", "UNBLOCK_MAC_STATUS: undefined BLOCK_MAC_COMMAND\n"); 
    return -1;
  }

  $sock->cmd("config terminal");
  my $command = $BLOCK{'BLOCK_MAC_COMMAND'};
  $command =~ s/ drop//g;	# cut  " drop"
  my $line = sock_cmd("no $command");
  &write2file("$request_file", "UNBLOCK_MAC_STATUS: $line\n");
  $sock->cmd("exit");
  return 1;
}

sub block_ip_cisco{
  my ($ip, $vlan) = @_;
  my $acl_name = "in_$vlan";
  my $line;

  $sock->cmd("config terminal");
  $sock->cmd("interface $vlan");
  &write2file("$request_file", "TARGET_VLAN: $vlan\n");
  $line = sock_cmd("ip access-group $acl_name in");
  &write2file("$request_file", "BLOCK_IP_STATUS: $line\n");
  $sock->cmd("exit");

  $line = sock_cmd("ip access-list extended $acl_name");
  &write2file("$request_file", "BLOCK_IP_STATUS: $line\n");
  $sock->cmd("deny ip host $ip any");
  $sock->cmd("no permit ip any any");
  $sock->cmd("permit ip any any");
  $sock->cmd("exit");
  $sock->cmd("exit");
  
  &write2file("$request_file", "TARGET_ACL: $acl_name\n");

  if ( -f $block_dir/$ip ) {
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},D";
    return 0;
  } else {
    &get_time;
    &write2file("$request_file", "BLOCK_IP_TIME: $my_time\n");
    &write2file("$request_file", "BLOCK_IP_TIMESTAMP: $my_timestamp\n");

    system("/bin/cp $request_file $block_dir/$ip");
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},S";
    return 1;
  }
}

sub unblock_ip_cisco{
  unless ( defined $BLOCK{'TARGET_ACL'} ) {
    &write2file("$request_file", "UNBLOCK_IP_STATUS: undefined ACL $BLOCK{'TARGET_ACL'}\n"); 
    rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},F";
    return -1;
  }

  $sock->cmd("config terminal");
  my $line = sock_cmd("ip access-list extended $BLOCK{'TARGET_ACL'}");
  &write2file("$request_file", "UNBLOCK_IP_STATUS: $line\n");
  $line = sock_cmd("no deny ip host $BLOCK{'REQUEST_TARGET'} any");
  &write2file("$request_file", "UNBLOCK_IP_STATUS: $line\n");
  $sock->cmd("exit");
  $sock->cmd("exit");

  &get_time;
  &write2file("$block_file", "UNBLOCK_IP_TIME: $my_time\n");
  &write2file("$block_file", "UNBLOCK_IP_TIMESTAMP: $my_timestamp\n");

  system("/bin/mv $block_file $unblock_dir/$BLOCK{'REQUEST_TARGET'},$my_time");
  rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},S";
  return 1;
}

sub block_ip_extreme{
  my ($ip) = @_;
  my $precedence = 101;

  # find percedence
  my @rules = $sock->cmd("show access-list");
#  Rule          Dest/mask:L4DP          Src/mask:L4SP       Flags  Hits
#accept-rip2       224.0.0.9/32: 520     0.0.0.0/ 0: 520     U-P-N   7109
#deny-140.114.75.20  0.0.0.0/ 0:   0     140.114.75.20/32:  0  I-D-X 789886
  foreach (@rules) {
    if (/(\S+)\s+\d+\.\d+\.\d+\.\d+\//) {
      #print "Rule: $1\n";
      $line = sock_cmd("show access-list $1");
      #print "$line\n";
# Precedence: 101
      if ($line =~ /Precedence:\s+(\d+)\s/) {
        #print "Precedence: $1\n";
        if ( $precedence > $1 ) {
          next;
        } elsif ( $precedence == $1 ) {
          $precedence++;
          next;
        } else {
          print "Selected precedence: $precedence\n";
          last;
        }
      }
    }
  }

#RETRY:
  my $acl_name = "deny-$ip-p$precedence";
  my $command = "create access-list $acl_name ip dest any sou $ip/32 deny port any precedence $precedence";
  $line = sock_cmd("$command");
  &write2file("$request_file", "BLOCK_IP_COMMAND: $command\n");
  &write2file("$request_file", "BLOCK_IP_STATUS: $line\n");
  #debug("$line\n");
  if ($line =~ /ERROR: This rule already exists/i) {
    #rename $request_file, "$archive_dir/$REQUEST{'REQUEST_ID'},D";
    return 1;
  } elsif ($line =~ /error/i) {
#No available rule number in slot 8 for prec 349 1 0 0 255
#ERROR: Failed to create rule deny-140.114.229.1-p349
    return -1, $line;
  }else {
    &write2file("$request_file", "TARGET_ACL: $acl_name\n");
    #&get_time;
    #&write2file("$request_file", "BLOCK_IP_TIME: $my_time\n");
    #&write2file("$request_file", "BLOCK_IP_TIMESTAMP: $my_timestamp\n");
    return 1;
  }
}

sub unblock_ip_extreme{

  # undefined ACL does not need to do
  unless ( defined $BLOCK{'TARGET_ACL'} ) {
    &write2file("$request_file", "UNBLOCK_IP_STATUS: undefined ACL $BLOCK{'TARGET_ACL'}\n"); 
    return 1;
  }

  # delete access-list
  my $command = "delete access-list $BLOCK{'TARGET_ACL'}";
  my $line = sock_cmd("$command");
  &write2file("$request_file", "UNBLOCK_IP_COMMAND: $command\n");
  &write2file("$request_file", "UNBLOCK_IP_STATUS: $line\n");
  return 1 if ($line =~ /Syntax error at token $BLOCK{'TARGET_ACL'}/);

  # double check by the same command
  $line = sock_cmd("$command");
  &write2file("$request_file", "UNBLOCK_IP_COMMAND2: $command\n");
  &write2file("$request_file", "UNBLOCK_IP_STATUS2: $line\n");
  if ($line =~ /Syntax error at token $BLOCK{'TARGET_ACL'}/) {
    #&write2file("$block_file", "UNBLOCK_IP_TIME: $my_time\n");
    #&write2file("$block_file", "UNBLOCK_IP_TIMESTAMP: $my_timestamp\n");
    return 1;
  } else {
    return -1;
  }
}

sub block_mac_extreme{
  my ($ip, $vlan) = @_;
  my $mac;
  $sock->cmd("ping $target");
  my $line = sock_cmd("show iparp $ip");
  #print $line;
#Destination     Mac                Age Static  VLAN    [VID]   Port
#140.114.87.1    08:00:20:96:0D:E7    9   NO  ip-87     [4081]  3:30
  if ($line =~ /$target\s+(\w\w\:\w\w\:\w\w\:\w\w\:\w\w\:\w\w)\s+/) {
    $mac = $1;
    &write2file("$request_file", "TARGET_MAC: $mac\n");
    &write2file("$request_file", "TARGET_VLAN: $vlan\n");
    my $command="create fdbentry $mac $vlan blackhole both";
    $line = sock_cmd("$command");
    &write2file("$request_file", "BLOCK_MAC_COMMAND: $command\n");
    &write2file("$request_file", "BLOCK_MAC_STATUS: $line\n");
    if ($line =~ /Unable to /) {
      return -1;
    } else {
      return 1;
    }
  }
  &write2file("$request_file", "SHOW_IPARP_STATUS: NOT FOUND\n$line\n");
  return -2;
}

sub unblock_mac_extreme{
  if (defined $BLOCK{'TARGET_MAC'}) {
    # delete fdbentry
    my $command = "delete fdbentry $BLOCK{'TARGET_MAC'} $BLOCK{'TARGET_VLAN'}"; 
    my $line = sock_cmd("$command");
    &write2file("$request_file", "UNBLOCK_MAC_COMMAND: $command\n");
    &write2file("$request_file", "UNBLOCK_MAC_STATUS: $line\n");
    return 1 if ($line =~ /Unable to delete permanent entry/);

    # double check by the same command 
    #$command = "show fdb permanent"; 
    $line = sock_cmd("$command");
    &write2file("$request_file", "UNBLOCK_MAC_COMMAND2: $command\n");
    &write2file("$request_file", "UNBLOCK_MAC_STATUS2: $line\n");
    if ($line =~ /Unable to delete permanent entry/) {
      return 1;
    } else {
      return -1;
    }
  } else {
    # undefined MAC address does not need to do
    return 1; 
  }
}
