#!/usr/bin/perl -w
#
# Mu-Cheng Hsieh <mucheng@cc.nthu.edu.tw>
# 2003/10/12

# local configuration
my $home_dir = "/usr/local/netguard";
my $registrar_conf = "$home_dir/etc/registrar.conf";
my $request_dir = "$home_dir/spool/request";
my $book_dir = "$home_dir/spool/book";

my $password = "MyGod";
my $prompt = ">> ";
my $timeout = 180; # give the user 180 seconds to type a line
my $authorization = 1;	# authorization required

my ($my_timestamp, $my_time, $my_ymd, $my_hms);	# time variables
# command word 
my $command_block_ip="BLOCK_IP";
my $command_unblock_ip="UNBLOCK_IP";

# request parameters 
my ($request_id, $request_from, $request_action, $request_target, $request_reason, $request_comment);


#
# Net Server
#
package MyPackage;
use strict;
use vars qw(@ISA);
use Net::Server::Fork; # any personality will do
@ISA = qw(Net::Server::Fork);

MyPackage->run(conf_file=>"$registrar_conf");
exit;

# interactive
sub process_request {
  my $self = shift;
  ($request_from) = $self->{server}->{peeraddr};

  local $SIG{ALRM} = sub { die "timeout!\n" };

  # command parser
  while(1) {
    eval {
      alarm($timeout);
      last if ( command_parser() );
      alarm(0);
      next;
    };

    if( $@=~/timedout/i ){
      print STDOUT "\r\nTIMED OUT.\r\n";
      return -1;
    } 
  }
  alarm(0);
  return 0;
}

sub command_parser{
 
  while( <STDIN> ){
    s/\r?\n$//;	# filter the tailer

    # authorization
    if ($authorization) {
      if (/^$password$/) {
        print STDOUT "+OK pass $request_from.\r\n$prompt";
        $authorization = 0;	# pass
        return 0;
      } else {
        print STDOUT "-ERR authorization first.\r\n";
	return -1;
      }
    }

    #print STDOUT "Debug: [$_]\n";

    # command parser
    if (/^exit|quit|bye$/i) {
      print STDOUT "+OK bye\r\n";
      return 1;

    # block ip
    } elsif ( ($request_action, $request_target, $request_reason, $request_comment) =
               /^(BAN\_IP|BLOCK\_IP)\s+(\S+)\s+(\S+)\s+(\S+)$/){
      if ( check_ip_format($request_target) ) {
        print STDOUT "-ERR wrong ip format: $request_target\r\n$prompt";
      } else {
        $request_action = $command_block_ip;
        if ( &request_block ) {
          print STDOUT "-ERR block ip failure\r\n$prompt";
        } else {
          print STDOUT "+OK accept request $my_time\r\n$prompt";
        }
      }

    # unblock ip
    } elsif ( ($request_action, $request_target) = /^(FREE\_IP|UNBLOCK\_IP)\s+(\S+)$/) {
      if ( check_ip_format($request_target) ) {
        print STDOUT "-ERR wrong ip format: $request_target\r\n$prompt";
      } else {
        $request_action = $command_unblock_ip;
        if ( &request_unblock ) {
          print STDOUT "-ERR unblock ip failure\r\n$prompt";
        } else {
          print STDOUT "+OK accept request $my_time\r\n$prompt";
        }
      }

    # book unblock ip
    } elsif ( ($request_target) = /^BOOK\_UNBLOCK\_IP\s+(\S+)$/) {
      if ( check_ip_format($request_target) ) {
        print STDOUT "-ERR wrong ip format: $request_target\r\n$prompt";
      } else {
        $request_action = $command_unblock_ip;
        if ( &request_book_unblock ) {
          print STDOUT "-ERR book unblock ip failure\r\n$prompt";
        } else {
          print STDOUT "+OK accept request $my_time\r\n$prompt";
        }
      }

    # unknown command
    } else {
      print STDOUT "-ERR unkown command [$_] \r\n$prompt";
    }
    return 0;
  }
}

# subroutines
sub check_ip_format{
  my ($ip) = @_;
  #print STDOUT "check_ip_format: $ip\r\n";
  my $i;
  if ($ip =~ /^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
    foreach $i ($1, $2, $3, $4) {
      if ($i < 0 || $i > 255 ) {
        return -1;
      }
    }
  } else {
    return -1;
  }
  return 0;   
}

sub check_subnet_format{
  my ($subnet) = @_;
  my $i;
  if ($subnet =~ /^(\d+)\.(\d+)\.(\d+)$/) {
    foreach $i ($1, $2, $3) {
      if ($i < 0 || $i > 255 ) {
        return -1;
      }
    }
  } else {
    return -1;
  }
  return 0;   
}

sub request_block{
  &get_time();
  my $request_file = "$my_time,$request_action,$request_target,$request_reason";
  open RQ_FH, "> $request_dir/$request_file" or return -1;
  print RQ_FH "REQUEST_ID: $request_file\n";
  print RQ_FH "REQUEST_TIME: $my_time\n";
  print RQ_FH "REQUEST_DATE: $my_ymd $my_hms\n";
  print RQ_FH "REQUEST_TIMESTAMP: $my_timestamp\n";
  print RQ_FH "REQUEST_FROM: $request_from\n";
  print RQ_FH "REQUEST_ACTION: $request_action\n";
  print RQ_FH "REQUEST_TARGET: $request_target\n";
  print RQ_FH "REQUEST_REASON: $request_reason\n";
  print RQ_FH "REQUEST_COMMENT: $request_comment\n";
  close RQ_FH; 
  return 0;
}

sub request_book_unblock{
  &get_time();
  #my $request_file = "$my_time,$request_action,$request_target"; 
  my $request_file = "$request_target"; 
  if ( -f "$book_dir/$request_file" ) {
    print STDOUT "-ERR $request_target has booked\r\n";
    return -2;
  } else {
    open RQ_FH, "> $book_dir/$request_file" or return -1;
    print RQ_FH "REQUEST_ID: $request_file\n";
    print RQ_FH "REQUEST_TIME: $my_time\n";
    print RQ_FH "REQUEST_DATE: $my_ymd $my_hms\n";
    print RQ_FH "REQUEST_TIMESTAMP: $my_timestamp\n";
    print RQ_FH "REQUEST_FROM: $request_from\n";
    print RQ_FH "REQUEST_ACTION: $request_action\n";
    print RQ_FH "REQUEST_TARGET: $request_target\n";
    close RQ_FH; 
    return 0;
  }
}

sub request_unblock{
  &get_time();
  my $request_file = "$my_time,$request_action,$request_target"; 
  open RQ_FH, "> $request_dir/$request_file" or return -1;
  print RQ_FH "REQUEST_ID: $request_file\n";
  print RQ_FH "REQUEST_TIME: $my_time\n";
  print RQ_FH "REQUEST_DATE: $my_ymd $my_hms\n";
  print RQ_FH "REQUEST_TIMESTAMP: $my_timestamp\n";
  print RQ_FH "REQUEST_FROM: $request_from\n";
  print RQ_FH "REQUEST_ACTION: $request_action\n";
  print RQ_FH "REQUEST_TARGET: $request_target\n";
  close RQ_FH; 
  return 0;
}

sub get_time{
  use Time::HiRes qw(gettimeofday);
  my ($seconds, $microseconds) = split /\./, gettimeofday;
  $my_timestamp = $seconds;
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) 
     = localtime ($my_timestamp);
  $year+=1900;
  $mon++;
  $my_ymd = sprintf("%04d/%02d/%02d", $year, $mon, $mday);
  $my_hms = sprintf("%02d:%02d:%02d", $hour, $min, $sec);
  $my_time = sprintf("%04d%02d%02d.%8s.%s", $year, $mon, $mday, $my_hms, $microseconds);
}
